"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLicense = exports.License = void 0;
const license_sdk_1 = require("@n8n_io/license-sdk");
const Logger_1 = require("./Logger");
const config_1 = __importDefault(require("./config"));
const Db = __importStar(require("./Db"));
const constants_1 = require("./constants");
async function loadCertStr() {
    var _a;
    const databaseSettings = await Db.collections.Settings.findOne({
        where: {
            key: constants_1.SETTINGS_LICENSE_CERT_KEY,
        },
    });
    return (_a = databaseSettings === null || databaseSettings === void 0 ? void 0 : databaseSettings.value) !== null && _a !== void 0 ? _a : '';
}
async function saveCertStr(value) {
    await Db.collections.Settings.upsert({
        key: constants_1.SETTINGS_LICENSE_CERT_KEY,
        value,
        loadOnStartup: false,
    }, ['key']);
}
class License {
    constructor() {
        this.logger = (0, Logger_1.getLogger)();
    }
    async init(instanceId) {
        if (this.manager) {
            return;
        }
        const server = config_1.default.getEnv('license.serverUrl');
        const autoRenewEnabled = config_1.default.getEnv('license.autoRenewEnabled');
        const autoRenewOffset = config_1.default.getEnv('license.autoRenewOffset');
        try {
            this.manager = new license_sdk_1.LicenseManager({
                server,
                tenantId: config_1.default.getEnv('license.tenantId'),
                productIdentifier: `n8n-${constants_1.N8N_VERSION}`,
                autoRenewEnabled,
                autoRenewOffset,
                logger: this.logger,
                loadCertStr,
                saveCertStr,
                deviceFingerprint: () => instanceId,
            });
            await this.manager.initialize();
        }
        catch (e) {
            if (e instanceof Error) {
                this.logger.error('Could not initialize license manager sdk', e);
            }
        }
    }
    async activate(activationKey) {
        if (!this.manager) {
            return;
        }
        await this.manager.activate(activationKey);
    }
    async renew() {
        if (!this.manager) {
            return;
        }
        await this.manager.renew();
    }
    isFeatureEnabled(feature) {
        if (!this.manager) {
            return false;
        }
        return this.manager.hasFeatureEnabled(feature);
    }
    isSharingEnabled() {
        return this.isFeatureEnabled(constants_1.LICENSE_FEATURES.SHARING);
    }
    isLogStreamingEnabled() {
        return this.isFeatureEnabled(constants_1.LICENSE_FEATURES.LOG_STREAMING);
    }
    isLdapEnabled() {
        return this.isFeatureEnabled(constants_1.LICENSE_FEATURES.LDAP);
    }
    isSamlEnabled() {
        return this.isFeatureEnabled(constants_1.LICENSE_FEATURES.SAML);
    }
    isAdvancedExecutionFiltersEnabled() {
        return this.isFeatureEnabled(constants_1.LICENSE_FEATURES.ADVANCED_EXECUTION_FILTERS);
    }
    getCurrentEntitlements() {
        var _a, _b;
        return (_b = (_a = this.manager) === null || _a === void 0 ? void 0 : _a.getCurrentEntitlements()) !== null && _b !== void 0 ? _b : [];
    }
    getFeatureValue(feature, requireValidCert) {
        if (!this.manager) {
            return undefined;
        }
        return this.manager.getFeatureValue(feature, requireValidCert);
    }
    getManagementJwt() {
        if (!this.manager) {
            return '';
        }
        return this.manager.getManagementJwt();
    }
    getMainPlan() {
        if (!this.manager) {
            return undefined;
        }
        const entitlements = this.getCurrentEntitlements();
        if (!entitlements.length) {
            return undefined;
        }
        return entitlements.find((entitlement) => entitlement.productMetadata.terms.isMainPlan);
    }
    getTriggerLimit() {
        var _a;
        return ((_a = this.getFeatureValue('quota:activeWorkflows')) !== null && _a !== void 0 ? _a : -1);
    }
    getPlanName() {
        var _a;
        return ((_a = this.getFeatureValue('planName')) !== null && _a !== void 0 ? _a : 'Community');
    }
}
exports.License = License;
let licenseInstance;
function getLicense() {
    if (licenseInstance === undefined) {
        licenseInstance = new License();
    }
    return licenseInstance;
}
exports.getLicense = getLicense;
//# sourceMappingURL=License.js.map