"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WaitingWebhooks = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const Db = __importStar(require("./Db"));
const ResponseHelper = __importStar(require("./ResponseHelper"));
const WebhookHelpers = __importStar(require("./WebhookHelpers"));
const NodeTypes_1 = require("./NodeTypes");
const WorkflowExecuteAdditionalData = __importStar(require("./WorkflowExecuteAdditionalData"));
const UserManagementHelper_1 = require("./UserManagement/UserManagementHelper");
const typedi_1 = require("typedi");
class WaitingWebhooks {
    async executeWebhook(httpMethod, fullPath, req, res) {
        n8n_workflow_1.LoggerProxy.debug(`Received waiting-webhook "${httpMethod}" for path "${fullPath}"`);
        req.params = {};
        if (fullPath.endsWith('/')) {
            fullPath = fullPath.slice(0, -1);
        }
        const pathParts = fullPath.split('/');
        const executionId = pathParts.shift();
        const path = pathParts.join('/');
        const execution = await Db.collections.Execution.findOneBy({ id: executionId });
        if (execution === null) {
            throw new ResponseHelper.NotFoundError(`The execution "${executionId} does not exist.`);
        }
        const fullExecutionData = ResponseHelper.unflattenExecutionData(execution);
        if (fullExecutionData.finished || fullExecutionData.data.resultData.error) {
            throw new ResponseHelper.ConflictError(`The execution "${executionId} has finished already.`);
        }
        return this.startExecution(httpMethod, path, fullExecutionData, req, res);
    }
    async startExecution(httpMethod, path, fullExecutionData, req, res) {
        const executionId = fullExecutionData.id;
        if (fullExecutionData.finished) {
            throw new Error('The execution did succeed and can so not be started again.');
        }
        const lastNodeExecuted = fullExecutionData.data.resultData.lastNodeExecuted;
        fullExecutionData.data.executionData.nodeExecutionStack[0].node.disabled = true;
        fullExecutionData.data.waitTill = undefined;
        fullExecutionData.data.resultData.runData[lastNodeExecuted].pop();
        const { workflowData } = fullExecutionData;
        const nodeTypes = typedi_1.Container.get(NodeTypes_1.NodeTypes);
        const workflow = new n8n_workflow_1.Workflow({
            id: workflowData.id.toString(),
            name: workflowData.name,
            nodes: workflowData.nodes,
            connections: workflowData.connections,
            active: workflowData.active,
            nodeTypes,
            staticData: workflowData.staticData,
            settings: workflowData.settings,
        });
        let workflowOwner;
        try {
            workflowOwner = await (0, UserManagementHelper_1.getWorkflowOwner)(workflowData.id.toString());
        }
        catch (error) {
            throw new ResponseHelper.NotFoundError('Could not find workflow');
        }
        const additionalData = await WorkflowExecuteAdditionalData.getBase(workflowOwner.id);
        const webhookData = n8n_workflow_1.NodeHelpers.getNodeWebhooks(workflow, workflow.getNode(lastNodeExecuted), additionalData).filter((webhook) => {
            return (webhook.httpMethod === httpMethod &&
                webhook.path === path &&
                webhook.webhookDescription.restartWebhook === true);
        })[0];
        if (webhookData === undefined) {
            const errorMessage = `The execution "${executionId}" with webhook suffix path "${path}" is not known.`;
            throw new ResponseHelper.NotFoundError(errorMessage);
        }
        const workflowStartNode = workflow.getNode(lastNodeExecuted);
        if (workflowStartNode === null) {
            throw new ResponseHelper.NotFoundError('Could not find node to process webhook.');
        }
        const runExecutionData = fullExecutionData.data;
        return new Promise((resolve, reject) => {
            const executionMode = 'webhook';
            WebhookHelpers.executeWebhook(workflow, webhookData, workflowData, workflowStartNode, executionMode, undefined, runExecutionData, fullExecutionData.id, req, res, (error, data) => {
                if (error !== null) {
                    return reject(error);
                }
                resolve(data);
            });
        });
    }
}
exports.WaitingWebhooks = WaitingWebhooks;
//# sourceMappingURL=WaitingWebhooks.js.map