"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OwnerController = void 0;
const validator_1 = __importDefault(require("validator"));
const GenericHelpers_1 = require("../GenericHelpers");
const decorators_1 = require("../decorators");
const ResponseHelper_1 = require("../ResponseHelper");
const UserManagementHelper_1 = require("../UserManagement/UserManagementHelper");
const jwt_1 = require("../auth/jwt");
let OwnerController = class OwnerController {
    constructor({ config, logger, internalHooks, repositories, }) {
        this.config = config;
        this.logger = logger;
        this.internalHooks = internalHooks;
        this.userRepository = repositories.User;
        this.settingsRepository = repositories.Settings;
        this.credentialsRepository = repositories.Credentials;
        this.workflowsRepository = repositories.Workflow;
    }
    async preSetup() {
        if (this.config.getEnv('userManagement.isInstanceOwnerSetUp')) {
            throw new ResponseHelper_1.BadRequestError('Instance owner already setup');
        }
        const [credentials, workflows] = await Promise.all([
            this.credentialsRepository.countBy({}),
            this.workflowsRepository.countBy({}),
        ]);
        return { credentials, workflows };
    }
    async setupOwner(req, res) {
        const { email, firstName, lastName, password } = req.body;
        const { id: userId, globalRole } = req.user;
        if (this.config.getEnv('userManagement.isInstanceOwnerSetUp')) {
            this.logger.debug('Request to claim instance ownership failed because instance owner already exists', {
                userId,
            });
            throw new ResponseHelper_1.BadRequestError('Instance owner already setup');
        }
        if (!email || !validator_1.default.isEmail(email)) {
            this.logger.debug('Request to claim instance ownership failed because of invalid email', {
                userId,
                invalidEmail: email,
            });
            throw new ResponseHelper_1.BadRequestError('Invalid email address');
        }
        const validPassword = (0, UserManagementHelper_1.validatePassword)(password);
        if (!firstName || !lastName) {
            this.logger.debug('Request to claim instance ownership failed because of missing first name or last name in payload', { userId, payload: req.body });
            throw new ResponseHelper_1.BadRequestError('First and last names are mandatory');
        }
        if (globalRole.scope === 'global' && globalRole.name !== 'owner') {
            this.logger.debug('Request to claim instance ownership failed because user shell does not exist or has wrong role!', {
                userId,
            });
            throw new ResponseHelper_1.BadRequestError('Invalid request');
        }
        let owner = req.user;
        Object.assign(owner, {
            email,
            firstName,
            lastName,
            password: await (0, UserManagementHelper_1.hashPassword)(validPassword),
        });
        await (0, GenericHelpers_1.validateEntity)(owner);
        owner = await this.userRepository.save(owner);
        this.logger.info('Owner was set up successfully', { userId });
        await this.settingsRepository.update({ key: 'userManagement.isInstanceOwnerSetUp' }, { value: JSON.stringify(true) });
        this.config.set('userManagement.isInstanceOwnerSetUp', true);
        this.logger.debug('Setting isInstanceOwnerSetUp updated successfully', { userId });
        await (0, jwt_1.issueCookie)(res, owner);
        void this.internalHooks.onInstanceOwnerSetup({ user_id: userId });
        return (0, UserManagementHelper_1.sanitizeUser)(owner);
    }
    async skipSetup() {
        await this.settingsRepository.update({ key: 'userManagement.skipInstanceOwnerSetup' }, { value: JSON.stringify(true) });
        this.config.set('userManagement.skipInstanceOwnerSetup', true);
        return { success: true };
    }
};
__decorate([
    (0, decorators_1.Get)('/pre-setup'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], OwnerController.prototype, "preSetup", null);
__decorate([
    (0, decorators_1.Post)('/setup'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], OwnerController.prototype, "setupOwner", null);
__decorate([
    (0, decorators_1.Post)('/skip-setup'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], OwnerController.prototype, "skipSetup", null);
OwnerController = __decorate([
    (0, decorators_1.RestController)('/owner'),
    __metadata("design:paramtypes", [Object])
], OwnerController);
exports.OwnerController = OwnerController;
//# sourceMappingURL=owner.controller.js.map