"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PurgeInvalidWorkflowConnections1675940580449 = void 0;
const migrationHelpers_1 = require("../../utils/migrationHelpers");
const NodeTypes_1 = require("../../../NodeTypes");
const Logger_1 = require("../../../Logger");
const typedi_1 = require("typedi");
class PurgeInvalidWorkflowConnections1675940580449 {
    constructor() {
        this.name = 'PurgeInvalidWorkflowConnections1675940580449';
    }
    async up(queryRunner) {
        (0, migrationHelpers_1.logMigrationStart)(this.name);
        const tablePrefix = (0, migrationHelpers_1.getTablePrefix)();
        const workflows = await queryRunner.query(`
			SELECT id, nodes, connections
			FROM "${tablePrefix}workflow_entity"
		`);
        const nodeTypes = typedi_1.Container.get(NodeTypes_1.NodeTypes);
        workflows.forEach(async (workflow) => {
            let connections = workflow.connections;
            const nodes = workflow.nodes;
            const nodesThatCannotReceiveInput = nodes.reduce((acc, node) => {
                var _a, _b;
                try {
                    const nodeType = nodeTypes.getByNameAndVersion(node.type, node.typeVersion);
                    if (((_b = (_a = nodeType.description.inputs) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : []) === 0) {
                        acc.push(node.name);
                    }
                }
                catch (error) {
                    (0, Logger_1.getLogger)().warn(`Migration ${this.name} failed with error: ${error.message}`);
                }
                return acc;
            }, []);
            Object.keys(connections).forEach((sourceNodeName) => {
                const connection = connections[sourceNodeName];
                const outputs = Object.keys(connection);
                outputs.forEach((outputConnectionName, idx) => {
                    const outputConnection = connection[outputConnectionName];
                    outputConnection.forEach((outputConnectionItem, outputConnectionItemIdx) => {
                        outputConnection[outputConnectionItemIdx] = outputConnectionItem.filter((outgoingConnections) => !nodesThatCannotReceiveInput.includes(outgoingConnections.node));
                    });
                });
            });
            const [updateQuery, updateParams] = queryRunner.connection.driver.escapeQueryWithParameters(`
						UPDATE "${tablePrefix}workflow_entity"
						SET connections = :connections
						WHERE id = '${workflow.id}'
					`, { connections: JSON.stringify(connections) }, {});
            await queryRunner.query(updateQuery, updateParams);
        });
        (0, migrationHelpers_1.logMigrationEnd)(this.name);
    }
    async down(queryRunner) {
    }
}
exports.PurgeInvalidWorkflowConnections1675940580449 = PurgeInvalidWorkflowConnections1675940580449;
//# sourceMappingURL=1675940580449-PurgeInvalidWorkflowConnections.js.map