"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UniqueWorkflowNames1620821879465 = void 0;
const config_1 = __importDefault(require("../../../config"));
const migrationHelpers_1 = require("../../utils/migrationHelpers");
class UniqueWorkflowNames1620821879465 {
    constructor() {
        this.name = 'UniqueWorkflowNames1620821879465';
    }
    async up(queryRunner) {
        (0, migrationHelpers_1.logMigrationStart)(this.name);
        const tablePrefix = config_1.default.getEnv('database.tablePrefix');
        const workflowNames = await queryRunner.query(`
				SELECT name
				FROM "${tablePrefix}workflow_entity"
			`);
        for (const { name } of workflowNames) {
            const [duplicatesQuery, parameters] = queryRunner.connection.driver.escapeQueryWithParameters(`
					SELECT id, name
					FROM "${tablePrefix}workflow_entity"
					WHERE name = :name
					ORDER BY createdAt ASC
				`, { name }, {});
            const duplicates = await queryRunner.query(duplicatesQuery, parameters);
            if (duplicates.length > 1) {
                await Promise.all(duplicates.map(({ id, name }, index) => {
                    if (index === 0)
                        return Promise.resolve();
                    const [updateQuery, updateParams] = queryRunner.connection.driver.escapeQueryWithParameters(`
							UPDATE "${tablePrefix}workflow_entity"
							SET name = :name
							WHERE id = '${id}'
						`, { name: `${name} ${index + 1}` }, {});
                    return queryRunner.query(updateQuery, updateParams);
                }));
            }
        }
        await queryRunner.query(`CREATE UNIQUE INDEX "IDX_${tablePrefix}943d8f922be094eb507cb9a7f9" ON "${tablePrefix}workflow_entity" ("name") `);
        (0, migrationHelpers_1.logMigrationEnd)(this.name);
    }
    async down(queryRunner) {
        const tablePrefix = config_1.default.getEnv('database.tablePrefix');
        await queryRunner.query(`DROP INDEX "IDX_${tablePrefix}943d8f922be094eb507cb9a7f9"`);
    }
}
exports.UniqueWorkflowNames1620821879465 = UniqueWorkflowNames1620821879465;
//# sourceMappingURL=1620821879465-UniqueWorkflowNames.js.map