"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoveWorkflowDataLoadedFlag1671726148419 = void 0;
const migrationHelpers_1 = require("../../utils/migrationHelpers");
const config_1 = __importDefault(require("../../../config"));
const WorkflowStatistics_1 = require("../../entities/WorkflowStatistics");
class RemoveWorkflowDataLoadedFlag1671726148419 {
    constructor() {
        this.name = 'RemoveWorkflowDataLoadedFlag1671726148419';
    }
    async up(queryRunner) {
        (0, migrationHelpers_1.logMigrationStart)(this.name);
        const tablePrefix = config_1.default.getEnv('database.tablePrefix');
        const workflowIds = await queryRunner.query(`
			SELECT id, dataLoaded
			FROM "${tablePrefix}workflow_entity"
		`);
        workflowIds.map(({ id, dataLoaded }) => {
            if (dataLoaded) {
                const [insertQuery, insertParams] = queryRunner.connection.driver.escapeQueryWithParameters(`
					INSERT INTO "${tablePrefix}workflow_statistics" (workflowId, name, count, latestEvent) VALUES
					(:id, :name, 1, STRFTIME('%Y-%m-%d %H:%M:%f', 'NOW'))
					`, { id, name: WorkflowStatistics_1.StatisticsNames.dataLoaded }, {});
                return queryRunner.query(insertQuery, insertParams);
            }
            return undefined;
        });
        await queryRunner.query(`ALTER TABLE \`${tablePrefix}workflow_entity\` DROP COLUMN "dataLoaded"`);
        (0, migrationHelpers_1.logMigrationEnd)(this.name);
    }
    async down(queryRunner) {
        const tablePrefix = config_1.default.getEnv('database.tablePrefix');
        await queryRunner.query(`ALTER TABLE \`${tablePrefix}workflow_entity\` ADD COLUMN "dataLoaded" BOOLEAN DEFAULT false`);
        const workflowsIds = await queryRunner.query(`
			SELECT workflowId
			FROM "${tablePrefix}workflow_statistics"
			WHERE name = '${WorkflowStatistics_1.StatisticsNames.dataLoaded}'
		`);
        workflowsIds.map(({ workflowId }) => {
            return queryRunner.query(`
				UPDATE "${tablePrefix}workflow_entity"
				SET dataLoaded = true
				WHERE id = '${workflowId}'`);
        });
        await queryRunner.query(`DELETE FROM "${tablePrefix}workflow_statistics" WHERE name = '${WorkflowStatistics_1.StatisticsNames.dataLoaded}'`);
    }
}
exports.RemoveWorkflowDataLoadedFlag1671726148419 = RemoveWorkflowDataLoadedFlag1671726148419;
//# sourceMappingURL=1671726148419-RemoveWorkflowDataLoadedFlag.js.map