"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerController = void 0;
const express_1 = require("express");
const constants_1 = require("./constants");
const ResponseHelper_1 = require("../ResponseHelper");
const registerController = (app, config, controller) => {
    var _a;
    const controllerClass = controller.constructor;
    const controllerBasePath = Reflect.getMetadata(constants_1.CONTROLLER_BASE_PATH, controllerClass);
    if (!controllerBasePath)
        throw new Error(`${controllerClass.name} is missing the RestController decorator`);
    const routes = Reflect.getMetadata(constants_1.CONTROLLER_ROUTES, controllerClass);
    if (routes.length > 0) {
        const router = (0, express_1.Router)({ mergeParams: true });
        const restBasePath = config.getEnv('endpoints.rest');
        const prefix = `/${[restBasePath, controllerBasePath].join('/')}`.replace(/\/+/g, '/');
        const controllerMiddlewares = ((_a = Reflect.getMetadata(constants_1.CONTROLLER_MIDDLEWARES, controllerClass)) !== null && _a !== void 0 ? _a : []).map(({ handlerName }) => controller[handlerName].bind(controller));
        routes.forEach(({ method, path, middlewares: routeMiddlewares, handlerName }) => {
            router[method](path, ...controllerMiddlewares, ...routeMiddlewares, (0, ResponseHelper_1.send)(async (req, res) => controller[handlerName](req, res)));
        });
        app.use(prefix, router);
    }
};
exports.registerController = registerController;
//# sourceMappingURL=registerController.js.map