"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.eventBusRouter = void 0;
const express_1 = __importDefault(require("express"));
const AbstractEventMessage_1 = require("./EventMessageClasses/AbstractEventMessage");
const EventMessageGeneric_1 = require("./EventMessageClasses/EventMessageGeneric");
const EventMessageWorkflow_1 = require("./EventMessageClasses/EventMessageWorkflow");
const MessageEventBus_1 = require("./MessageEventBus/MessageEventBus");
const MessageEventBusDestinationSentry_ee_1 = require("./MessageEventBusDestination/MessageEventBusDestinationSentry.ee");
const MessageEventBusDestinationSyslog_ee_1 = require("./MessageEventBusDestination/MessageEventBusDestinationSyslog.ee");
const MessageEventBusDestinationWebhook_ee_1 = require("./MessageEventBusDestination/MessageEventBusDestinationWebhook.ee");
const EventMessageClasses_1 = require("./EventMessageClasses");
const EventMessageAudit_1 = require("./EventMessageClasses/EventMessageAudit");
const ResponseHelper_1 = require("../ResponseHelper");
const n8n_workflow_1 = require("n8n-workflow");
const ResponseHelper = __importStar(require("../ResponseHelper"));
const EventMessageNode_1 = require("./EventMessageClasses/EventMessageNode");
const recoverEvents_1 = require("./MessageEventBus/recoverEvents");
exports.eventBusRouter = express_1.default.Router();
const isWithIdString = (candidate) => {
    const o = candidate;
    if (!o)
        return false;
    return o.id !== undefined;
};
const isWithQueryString = (candidate) => {
    const o = candidate;
    if (!o)
        return false;
    return o.query !== undefined;
};
const isMessageEventBusDestinationWebhookOptions = (candidate) => {
    const o = candidate;
    if (!o)
        return false;
    return o.url !== undefined;
};
const isMessageEventBusDestinationOptions = (candidate) => {
    const o = candidate;
    if (!o)
        return false;
    return o.__type !== undefined;
};
exports.eventBusRouter.get('/event', ResponseHelper.send(async (req) => {
    if (isWithQueryString(req.query)) {
        switch (req.query.query) {
            case 'sent':
                return MessageEventBus_1.eventBus.getEventsSent();
            case 'unsent':
                return MessageEventBus_1.eventBus.getEventsUnsent();
            case 'unfinished':
                return MessageEventBus_1.eventBus.getUnfinishedExecutions();
            case 'all':
            default:
                return MessageEventBus_1.eventBus.getEventsAll();
        }
    }
    return MessageEventBus_1.eventBus.getEventsAll();
}));
exports.eventBusRouter.get('/execution/:id', ResponseHelper.send(async (req) => {
    var _a, _b;
    if ((_a = req.params) === null || _a === void 0 ? void 0 : _a.id) {
        let logHistory;
        if ((_b = req.query) === null || _b === void 0 ? void 0 : _b.logHistory) {
            logHistory = parseInt(req.query.logHistory, 10);
        }
        return MessageEventBus_1.eventBus.getEventsByExecutionId(req.params.id, logHistory);
    }
}));
exports.eventBusRouter.get('/execution-recover/:id', ResponseHelper.send(async (req) => {
    var _a, _b, _c;
    if ((_a = req.params) === null || _a === void 0 ? void 0 : _a.id) {
        let logHistory;
        let applyToDb = true;
        if ((_b = req.query) === null || _b === void 0 ? void 0 : _b.logHistory) {
            logHistory = parseInt(req.query.logHistory, 10);
        }
        if ((_c = req.query) === null || _c === void 0 ? void 0 : _c.applyToDb) {
            applyToDb = !!req.query.applyToDb;
        }
        const messages = await MessageEventBus_1.eventBus.getEventsByExecutionId(req.params.id, logHistory);
        if (messages.length > 0) {
            const recoverResult = await (0, recoverEvents_1.recoverExecutionDataFromEventLogMessages)(req.params.id, messages, applyToDb);
            return recoverResult;
        }
    }
}));
exports.eventBusRouter.post('/event', ResponseHelper.send(async (req) => {
    if ((0, AbstractEventMessage_1.isEventMessageOptions)(req.body)) {
        let msg;
        switch (req.body.__type) {
            case n8n_workflow_1.EventMessageTypeNames.workflow:
                msg = new EventMessageWorkflow_1.EventMessageWorkflow(req.body);
                break;
            case n8n_workflow_1.EventMessageTypeNames.audit:
                msg = new EventMessageAudit_1.EventMessageAudit(req.body);
                break;
            case n8n_workflow_1.EventMessageTypeNames.node:
                msg = new EventMessageNode_1.EventMessageNode(req.body);
                break;
            case n8n_workflow_1.EventMessageTypeNames.generic:
            default:
                msg = new EventMessageGeneric_1.EventMessageGeneric(req.body);
        }
        await MessageEventBus_1.eventBus.send(msg);
    }
    else {
        throw new ResponseHelper_1.BadRequestError('Body is not a serialized EventMessage or eventName does not match format {namespace}.{domain}.{event}');
    }
}));
exports.eventBusRouter.get('/destination', ResponseHelper.send(async (req) => {
    let result = [];
    if (isWithIdString(req.query)) {
        result = await MessageEventBus_1.eventBus.findDestination(req.query.id);
    }
    else {
        result = await MessageEventBus_1.eventBus.findDestination();
    }
    return result;
}));
exports.eventBusRouter.post('/destination', ResponseHelper.send(async (req) => {
    if (!req.user || req.user.globalRole.name !== 'owner') {
        throw new ResponseHelper.UnauthorizedError('Invalid request');
    }
    if (isMessageEventBusDestinationOptions(req.body)) {
        let result;
        switch (req.body.__type) {
            case n8n_workflow_1.MessageEventBusDestinationTypeNames.sentry:
                if ((0, MessageEventBusDestinationSentry_ee_1.isMessageEventBusDestinationSentryOptions)(req.body)) {
                    result = await MessageEventBus_1.eventBus.addDestination(new MessageEventBusDestinationSentry_ee_1.MessageEventBusDestinationSentry(MessageEventBus_1.eventBus, req.body));
                }
                break;
            case n8n_workflow_1.MessageEventBusDestinationTypeNames.webhook:
                if (isMessageEventBusDestinationWebhookOptions(req.body)) {
                    result = await MessageEventBus_1.eventBus.addDestination(new MessageEventBusDestinationWebhook_ee_1.MessageEventBusDestinationWebhook(MessageEventBus_1.eventBus, req.body));
                }
                break;
            case n8n_workflow_1.MessageEventBusDestinationTypeNames.syslog:
                if ((0, MessageEventBusDestinationSyslog_ee_1.isMessageEventBusDestinationSyslogOptions)(req.body)) {
                    result = await MessageEventBus_1.eventBus.addDestination(new MessageEventBusDestinationSyslog_ee_1.MessageEventBusDestinationSyslog(MessageEventBus_1.eventBus, req.body));
                }
                break;
            default:
                throw new ResponseHelper_1.BadRequestError(`Body is missing ${req.body.__type} options or type ${req.body.__type} is unknown`);
        }
        if (result) {
            await result.saveToDb();
            return {
                ...result,
                eventBusInstance: undefined,
            };
        }
        throw new ResponseHelper_1.BadRequestError('There was an error adding the destination');
    }
    throw new ResponseHelper_1.BadRequestError('Body is not configuring MessageEventBusDestinationOptions');
}));
exports.eventBusRouter.get('/testmessage', ResponseHelper.send(async (req) => {
    let result = false;
    if (isWithIdString(req.query)) {
        result = await MessageEventBus_1.eventBus.testDestination(req.query.id);
    }
    return result;
}));
exports.eventBusRouter.delete('/destination', ResponseHelper.send(async (req) => {
    if (!req.user || req.user.globalRole.name !== 'owner') {
        throw new ResponseHelper.UnauthorizedError('Invalid request');
    }
    if (isWithIdString(req.query)) {
        const result = await MessageEventBus_1.eventBus.removeDestination(req.query.id);
        if (result) {
            return result;
        }
    }
    else {
        throw new ResponseHelper_1.BadRequestError('Query is missing id');
    }
}));
exports.eventBusRouter.get('/eventnames', ResponseHelper.send(async () => {
    return EventMessageClasses_1.eventNamesAll;
}));
//# sourceMappingURL=eventBusRoutes.js.map