"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodeFetchedData = exports.workflowExecutionCompleted = void 0;
const Db = __importStar(require("../Db"));
const WorkflowStatistics_1 = require("../databases/entities/WorkflowStatistics");
const UserManagementHelper_1 = require("../UserManagement/UserManagementHelper");
const typeorm_1 = require("typeorm");
const typedi_1 = require("typedi");
const InternalHooks_1 = require("../InternalHooks");
async function workflowExecutionCompleted(workflowData, runData) {
    const finished = runData.finished ? runData.finished : false;
    const manual = runData.mode === 'manual';
    let name;
    if (finished) {
        if (manual)
            name = WorkflowStatistics_1.StatisticsNames.manualSuccess;
        else
            name = WorkflowStatistics_1.StatisticsNames.productionSuccess;
    }
    else {
        if (manual)
            name = WorkflowStatistics_1.StatisticsNames.manualError;
        else
            name = WorkflowStatistics_1.StatisticsNames.productionError;
    }
    const workflowId = workflowData.id;
    if (!workflowId)
        return;
    try {
        await Db.collections.WorkflowStatistics.insert({
            count: 1,
            name,
            workflowId,
            latestEvent: new Date(),
        });
        if (name !== WorkflowStatistics_1.StatisticsNames.productionSuccess)
            return;
        const owner = await (0, UserManagementHelper_1.getWorkflowOwner)(workflowId);
        const metrics = {
            user_id: owner.id,
            workflow_id: workflowId,
        };
        await typedi_1.Container.get(InternalHooks_1.InternalHooks).onFirstProductionWorkflowSuccess(metrics);
    }
    catch (error) {
        if (!(error instanceof typeorm_1.QueryFailedError)) {
            throw error;
        }
        await Db.collections.WorkflowStatistics.update({ workflowId, name }, { count: () => 'count + 1', latestEvent: new Date() });
    }
}
exports.workflowExecutionCompleted = workflowExecutionCompleted;
async function nodeFetchedData(workflowId, node) {
    if (!workflowId)
        return;
    try {
        await Db.collections.WorkflowStatistics.insert({
            workflowId,
            name: WorkflowStatistics_1.StatisticsNames.dataLoaded,
            count: 1,
            latestEvent: new Date(),
        });
    }
    catch (error) {
        if (!(error instanceof typeorm_1.QueryFailedError)) {
            throw error;
        }
        return;
    }
    const owner = await (0, UserManagementHelper_1.getWorkflowOwner)(workflowId);
    let metrics = {
        user_id: owner.id,
        workflow_id: workflowId,
        node_type: node.type,
        node_id: node.id,
    };
    if (node.credentials) {
        Object.entries(node.credentials).forEach(([credName, credDetails]) => {
            metrics = Object.assign(metrics, {
                credential_type: credName,
                credential_id: credDetails.id,
            });
        });
    }
    await typedi_1.Container.get(InternalHooks_1.InternalHooks).onFirstWorkflowDataLoad(metrics);
}
exports.nodeFetchedData = nodeFetchedData;
//# sourceMappingURL=WorkflowStatistics.js.map