"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.samlControllerProtected = void 0;
const express_1 = __importDefault(require("express"));
const samlEnabledMiddleware_1 = require("../middleware/samlEnabledMiddleware");
const saml_service_ee_1 = require("../saml.service.ee");
const constants_1 = require("../constants");
const ResponseHelper_1 = require("../../../ResponseHelper");
const jwt_1 = require("../../../auth/jwt");
const class_validator_1 = require("class-validator");
const initSsoPost_1 = require("../views/initSsoPost");
const initSsoRedirect_1 = require("../views/initSsoRedirect");
exports.samlControllerProtected = express_1.default.Router();
exports.samlControllerProtected.get(constants_1.SamlUrls.config, samlEnabledMiddleware_1.samlLicensedOwnerMiddleware, (req, res) => {
    const prefs = saml_service_ee_1.SamlService.getInstance().samlPreferences;
    return res.send(prefs);
});
exports.samlControllerProtected.post(constants_1.SamlUrls.config, samlEnabledMiddleware_1.samlLicensedOwnerMiddleware, async (req, res) => {
    const validationResult = await (0, class_validator_1.validate)(req.body);
    if (validationResult.length === 0) {
        const result = await saml_service_ee_1.SamlService.getInstance().setSamlPreferences(req.body);
        return res.send(result);
    }
    else {
        throw new ResponseHelper_1.BadRequestError('Body is not a valid SamlPreferences object: ' +
            validationResult.map((e) => e.toString()).join(','));
    }
});
exports.samlControllerProtected.post(constants_1.SamlUrls.configToggleEnabled, samlEnabledMiddleware_1.samlLicensedOwnerMiddleware, async (req, res) => {
    if (req.body.loginEnabled === undefined) {
        throw new ResponseHelper_1.BadRequestError('Body should contain a boolean "loginEnabled" property');
    }
    await saml_service_ee_1.SamlService.getInstance().setSamlPreferences({ loginEnabled: req.body.loginEnabled });
    res.sendStatus(200);
});
exports.samlControllerProtected.get(constants_1.SamlUrls.acs, samlEnabledMiddleware_1.samlLicensedAndEnabledMiddleware, async (req, res) => {
    const loginResult = await saml_service_ee_1.SamlService.getInstance().handleSamlLogin(req, 'redirect');
    if (loginResult) {
        if (loginResult.authenticatedUser) {
            await (0, jwt_1.issueCookie)(res, loginResult.authenticatedUser);
            if (loginResult.onboardingRequired) {
                return res.redirect(constants_1.SamlUrls.samlOnboarding);
            }
            else {
                return res.redirect(constants_1.SamlUrls.defaultRedirect);
            }
        }
    }
    throw new ResponseHelper_1.AuthError('SAML Authentication failed');
});
exports.samlControllerProtected.post(constants_1.SamlUrls.acs, samlEnabledMiddleware_1.samlLicensedAndEnabledMiddleware, async (req, res) => {
    const loginResult = await saml_service_ee_1.SamlService.getInstance().handleSamlLogin(req, 'post');
    if (loginResult) {
        if (loginResult.authenticatedUser) {
            await (0, jwt_1.issueCookie)(res, loginResult.authenticatedUser);
            if (loginResult.onboardingRequired) {
                return res.redirect(constants_1.SamlUrls.samlOnboarding);
            }
            else {
                return res.redirect(constants_1.SamlUrls.defaultRedirect);
            }
        }
    }
    throw new ResponseHelper_1.AuthError('SAML Authentication failed');
});
exports.samlControllerProtected.get(constants_1.SamlUrls.initSSO, samlEnabledMiddleware_1.samlLicensedAndEnabledMiddleware, async (req, res) => {
    const result = saml_service_ee_1.SamlService.getInstance().getLoginRequestUrl();
    if ((result === null || result === void 0 ? void 0 : result.binding) === 'redirect') {
        return res.send((0, initSsoRedirect_1.getInitSSOPostView)(result.context));
    }
    else if ((result === null || result === void 0 ? void 0 : result.binding) === 'post') {
        return res.send((0, initSsoPost_1.getInitSSOFormView)(result.context));
    }
    else {
        throw new ResponseHelper_1.AuthError('SAML redirect failed, please check your SAML configuration.');
    }
});
exports.samlControllerProtected.get(constants_1.SamlUrls.configTest, samlEnabledMiddleware_1.samlLicensedOwnerMiddleware, async (req, res) => {
    const testResult = await saml_service_ee_1.SamlService.getInstance().testSamlConnection();
    return res.send(testResult);
});
//# sourceMappingURL=saml.controller.protected.ee.js.map